﻿#Requires -Version 3.0

#-----------------------------------------------------------------
# Copyright (c) 2017 Quest Software
#
# Name: Install.ps1 
# Description:
#    Installs new 'demo' schema on both a Source and Target Database           
#-----------------------------------------------------------------


# Initialize all Variables/Functions used in this process
. (Join-Path $PSScriptRoot '.\Config.ps1') 
Import-Module (Join-Path $PSScriptRoot '..\PSModules\LogUtils.psm1') -Force


# Start Transcript of DEMO Schema Install
$ErrorActionPreference = "SilentlyContinue"
Stop-Transcript        | out-null
$ErrorActionPreference = "Continue"
$LogRunDate     = Get-Date -Format yyyyMMddHHmmss
$LogFile        = 'OracleDemoSchemaInstall_Log.txt'
$LogPath        = "$LogDir\$LogFile"
Start-Transcript -path $LogPath -append


# Locate SQL*Plus
try {
  $SQLPlusBinary = (Get-Command -Name sqlplus.exe -ErrorAction Stop).Path
} catch {
  Show-Log -LogType Txt -LogText 'SQLPlus.exe could not be located. Install SQL*Plus and add its path to PATH environment variable, execute Install.sql from Toad, etc.'
  exit 1
}


$StartDateTime = Get-Date 
Show-Log -LogType Hdr -LogText 'Oracle DEMO Schema Install'

# Prompt for source DB info
Show-Log -LogType Txt -LogText 'Getting Source Database Login Information...'
$SourceDB             = Read-Host 'Enter Source Database Name'
$SourceUser           = Read-Host 'Enter Source Database Username (Requires SYSDBA privs)'
$SourcePasswordSecure = Read-Host -AsSecureString 'Enter Source Database Password'
$SourcePassword       = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($SourcePasswordSecure))



Write-Output ''

# Prompt for target DB info
Show-Log -LogType Txt -LogText 'Getting Target Database Login Information...'
$TargetDB             = Read-Host 'Enter Target Database Name'
$TargetUser           = Read-Host 'Enter Target Database Username (Requires SYSDBA privs)'
$TargetPasswordSecure = Read-Host -AsSecureString 'Enter Target Database Password'
$TargetPassword       = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($TargetPasswordSecure))


Write-Output ''


if ($SourceDB -ne $TargetDB)
   {# Confirm create of DEMO schema
    $Choice = ""
    while ($Choice -NotMatch "[Y|N]") 
       {$Confirmation = "DEMO schema will be created on $SourceDB and $TargetDB. Continue [Y/N]?"
        $Choice = Read-Host $Confirmation
       }
    
    if ($Choice -eq "N") 
       {exit 1}

    # Setup script params
    $Params = @(
    $SourceDB
    $SourceUser
    $SourcePassword
    $TargetDB
    $TargetUser
    $TargetPassword)

    # Run install.sql
    Show-Log -LogType Txt -LogText 'Running Install.sql'
    $Script = "$OracleDir\Install.sql"
    & $SQLPlusBinary -l $SourceUser/$SourcePassword@$SourceDB as SYSDBA @$Script $Params 2>&1

    if ($LastExitCode) 
       {Show-Log -LogType Txt -LogText 'Warning!!! Install.sql failed'
        exit 1
       }


    # Add Source and Target Database Names to Config.ps1 script
    Show-LogSeperator
    Show-Log -LogType Txt -LogText "Updating Config.ps1 script..."
    Show-Log -LogType Txt -LogText "    SourceDatabase = $SourceDB"
    Show-Log -LogType Txt -LogText "    TargetDatabase = $TargetDB"
    Show-LogSeperator

    $FileText = Get-Content "$PSScriptsDir\Config.ps1" -Raw
    $FileText = $FileText -Replace "(?ms)(^.*?\`$SourceDatabase\b[^']+')([^']+)(.*)", "`$1${SourceDB}`$3"
    $FileText = $FileText -Replace "(?ms)(^.*?\`$TargetDatabase\b[^']+')([^']+)(.*)", "`$1${TargetDB}`$3"
    $FileText | Out-File -FilePath "$PSScriptsDir\Config.ps1" -Force
   }
else
   {Show-LogSeperator
    Show-Log -LogType Txt -LogText 'WARNING!! Source and Target Databases must be different'
    Show-Log -LogType Txt -LogText ' - Rerun with different Source and Target DB Names.'
    Show-LogSeperator
    exit 1
   }


Show-Log -LogType End -LogText $StartDateTime
Stop-Transcript
Invoke-Item $LogPath
